// ----------------------------------------------------------------------------------
// Microsoft Developer & Platform Evangelism
// 
// Copyright (c) Microsoft Corporation. All rights reserved.
// 
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// ----------------------------------------------------------------------------------
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
// ----------------------------------------------------------------------------------

namespace PlanMyNight.Models
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Text;
    using PlanMyNight.Models.Caching;
    using PlanMyNight.Models.Entities;

    public class CachedActivitiesRepository : IActivitiesRepository
    {
        // Container Names
        private const string ReferenceDataCacheContainer = "Reference";
        private const string ActivityCacheContainer = "Activity";
        private const string SearchQueryCacheContainer = "ActivitySearch";

        private readonly ICachingProvider cacheProvider;

        private readonly IActivitiesRepository activitiesRepository;

        public CachedActivitiesRepository() :
            this(new VelocityProvider(), new ActivitiesRepository())
        {
        }

        public CachedActivitiesRepository(ICachingProvider cacheProvider, IActivitiesRepository activitiesRepository)
        {
            this.cacheProvider = cacheProvider;
            this.activitiesRepository = activitiesRepository;
        }

        public IEnumerable<string> RetrieveStates()
        {
            var cacheKey = "states";

            // retrieve states from Cache
            IEnumerable<string> data = this.cacheProvider.Get(ReferenceDataCacheContainer, cacheKey) as IEnumerable<string>;
            if (data != null)
            {
                return data;
            }

            // retrieve from Repository
            data = this.activitiesRepository.RetrieveStates();
            this.cacheProvider.Add(ReferenceDataCacheContainer, cacheKey, data, TimeSpan.FromMinutes(60));
            return data;
        }

        public Activity Retrieve(string id)
        {
            // retrieve from Cache; the Activity's id is used as key
            Activity data = this.cacheProvider.Get(ActivityCacheContainer, id) as Activity;
            if (data != null)
            {
                return data;
            }

            // retrieve from Repository
            data = this.activitiesRepository.Retrieve(id);
            this.cacheProvider.Add(ActivityCacheContainer, id, data, TimeSpan.FromMinutes(15));
            return data;
        }

        public IEnumerable<ActivityType> RetrieveActivityTypes()
        {
            var cacheKey = "activityTypes";

            // retrieve activityTypes from Cache
            IEnumerable<ActivityType> data = this.cacheProvider.Get(ReferenceDataCacheContainer, cacheKey) as IEnumerable<ActivityType>;
            if (data != null)
            {
                return data;
            }

            // retrieve from Repository
            data = this.activitiesRepository.RetrieveActivityTypes();
            this.cacheProvider.Add(ReferenceDataCacheContainer, cacheKey, data, TimeSpan.FromMinutes(60));
            return data;
        }

        public void RateActivity(string activityId, byte rating)
        {
            // Remove the activity from cache since it's invalid.
            this.cacheProvider.Remove(ActivityCacheContainer, activityId);

            this.activitiesRepository.RateActivity(activityId, rating);
        }

        public PagingResult<Activity> Search(ActivitySearchCriteria criteria)
        {
            var cacheKey = Serialize(criteria);

            // retrieve search results from Cache
            PagingResult<Activity> data = this.cacheProvider.Get(SearchQueryCacheContainer, cacheKey) as PagingResult<Activity>;
            if (data != null)
            {
                return data;
            }

            // retrieve from Repository
            data = this.activitiesRepository.Search(criteria);
            this.cacheProvider.Add(SearchQueryCacheContainer, cacheKey, data, TimeSpan.FromMinutes(5));

            // add activities to cache
            foreach (var activity in data.Items)
            {
                if (!this.cacheProvider.HasKey(ActivityCacheContainer, activity.Id))
                {
                    this.cacheProvider.Add(ActivityCacheContainer, activity.Id, activity, TimeSpan.FromMinutes(15));
                }
            }

            return data;
        }

        private static string Serialize(ActivitySearchCriteria criteria)
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.ActivityTypeId ?? 0);

            if (!string.IsNullOrEmpty(criteria.StreetAddress))
            {
                sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.StreetAddress);
            }

            sb.Append("|");
            if (!string.IsNullOrEmpty(criteria.State))
            {
                sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.State);
            }

            sb.Append("|");
            if (!string.IsNullOrEmpty(criteria.City))
            {
                sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.City);
            }

            sb.Append("|");
            if (!string.IsNullOrEmpty(criteria.Zip))
            {
                sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.Zip);
            }

            sb.Append("|");
            sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.StreetAddress);

            sb.Append("|");
            sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.SortBy);

            sb.Append("|");
            sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.Page);

            sb.Append("|");
            sb.AppendFormat(CultureInfo.InvariantCulture, "{0}", criteria.PageSize);

            return sb.ToString();
        }

    }
}